@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 
-from forwardable import Forwardable, def_delegators
-
-from .model_cache_class import *
+import os
+from .storage import *
+from .model_cache_class import ModelCacheClass
 
 valid_storage_types = ("memory", "sqlite", "redis")
 
@@ -14,7 +14,7 @@ class ModelCache():
 
         # setup args
         default_kwargs = {
-                    'cache_dir'      : None,
+                    'cache_dir'      : os.getenv("ModelCacheDir"),
                     'storage_type'   : 'sqlite',
                     'percentage'     : 0.9999,
                     'filter_lambda'  : lambda item1: False,
@@ -38,6 +38,11 @@ class ModelCache():
             # 1. included_class should not overwrite ModelCacheClass's important methods,
             #    include `__init__`, `init__before`, `init__after`.
             # 2. ensure decorated_class's methods will overwrite ModelCache's.
+
+            for k1 in ['init__before', 'init__after']:
+                if k1 in dir(default_kwargs['included_class']):
+                    setattr(ModelCacheClass, k1, getattr(default_kwargs['included_class'], k1))
+
             class _model_cache(decorated_class, ModelCacheClass, default_kwargs['included_class']):
                 class OriginalClass(): pass # so we can setattr here.
                 original = OriginalClass()
@@ -49,33 +54,38 @@ class ModelCache():
                 # Thx http://stackoverflow.com/questions/4932438/how-to-create-a-custom-string-representation-for-a-class-object/4932473#4932473
                 class MetaClass(type):
                     __metaclass__ = Forwardable
-                    _ = def_delegators('datadict', ('__getitem__', '__setitem__', '__delitem__', \
-                                                    '__iter__', '__len__', '__contains__', \
-                                                    'has_key', 'get', 'pop', 'popitem', \
-                                                    'keys', 'values', \
-                                                    'items', 'iteritems', 'iterkeys', 'itervalues', ))
+                    _ = def_delegators('datadict', dict_attrs)
 
                     def __repr__(self):
-                        while len(self.first_five_items) < 5:
+                        is_total_len_enough = len(self) > 5
+                        while is_total_len_enough and (len(self.first_five_items) < 5):
                             for item_id1, item1 in self.iteritems():
                                 self.first_five_items.append(item1)
                                 if len(self.first_five_items) == 5: break
 
-                        dots = ", ......" if len(self) > 5 else ""
+                        dots = ", ......" if is_total_len_enough else ""
                         return (u"<%s has %i items:[%s%s]>" % \
                                         (self.__name__, len(self), \
                                         ", ".join([str(item1.item_id) for item1 in self.first_five_items]), \
                                         dots, )).encode("UTF-8")
-
-
                 __metaclass__ = MetaClass
 
+                @classmethod
+                def pickle_path(cls, name):
+                    return cls.cache_dir + "/" + name + ".cPickle"
+
             _model_cache.__name__   = decorated_class.__name__
             _model_cache.__module__ = decorated_class.__module__ # so can pickle :)
 
             _model_cache.first_five_items = []
 
-            _model_cache.cache_dir  = cache_dir
+            _model_cache.cache_dir  = os.path.join(cache_dir or u"", _model_cache.__name__)
+            if default_kwargs['storage_type'] != 'memory':
+                if not os.path.isdir(_model_cache.cache_dir): os.makedirs(_model_cache.cache_dir)
+            _model_cache.dbpath = None
+            if _model_cache.cache_dir:
+                _model_cache.dbpath = os.path.join(_model_cache.cache_dir, _model_cache.__name__ + ".db")
+
             _model_cache.connect()
 
             return _model_cache
