@@ -1,9 +1,7 @@
 # -*- coding: utf-8 -*-
 
-import os
 import json
 from .storage import *
-from etl_utils import process_notifier
 
 class ModelCacheClass(object):
 
@@ -44,27 +42,28 @@ class ModelCacheClass(object):
 
     @classmethod
     def reconnect(cls, is_reconnect=True):
-        dbpath = None
-        if cls.cache_dir:
-            dbpath = os.path.join(cls.cache_dir, \
-                    cls.__name__ + ".db")
-
         cls.datadict = {
             "memory" : ModelCacheStoreMemory,
             "sqlite" : ModelCacheStoreSqlite,
             "redis"  : ModelCacheStoreRedis,
-        }[cls.original.storage_type](dbpath)
+        }[cls.original.storage_type](cls.dbpath)
 
         msg = 'Reconnect' if is_reconnect else 'Init'
-        print "[ModelCache] %s at %s" % (msg, dbpath or '[memory]')
+        print "[ModelCache] %s at %s" % (msg, cls.dbpath or '[memory]')
 
     @classmethod
     def pull_data(cls):
         print; print "[LOAD] %s [INTO] %s" % (cls.original.model.__module__, cls.__module__)
 
-        if len(cls) / float(cls.original.model.count()) < cls.original.percentage:
+        original_model_count = set_default_value([ \
+                lambda : cls.original.model.count(), \
+                lambda : len(cls.original.model), \
+                ], u"original.model is invalid!")
+
+        if len(cls) / float(original_model_count) < cls.original.percentage:
             print "[load ids cache] ..."
-            ids_cache = {str(i1.item_id) : True for i1 in process_notifier(cls.datadict.datadict)}
+            cls.datadict.sync() # sync first
+            ids_cache = {item_id1 : True for item_id1, item1 in process_notifier(cls.datadict.datadict)}
 
             items = []
             for e1 in process_notifier(cls.original.model):
@@ -83,5 +82,10 @@ class ModelCacheClass(object):
     def feed_data(cls, items=[]):
         # items 必定是list, 经过cPickle反序列化回来的
         """ 也许reopen在build_indexes解决sqlite close等问题 """
-        cls.datadict.feed_data(items)
+        for i1 in items:
+            # Fix InterfaceError: Error binding parameter 0 - probably unsupported type
+            assert isinstance(i1.item_id, (str, unicode, int,)), ("not " + repr(i1.item_id))
+
+            cls.datadict[i1.item_id] = i1
+
         cls.datadict.sync()
